clear;
close all;
warning('off')      %removes all the warnings coming from the fitting. They appear because data is not centered on 0 and not scaled.

%% Parameters
filename='Prelim-GM6-12-8-13.xlsx';         %provide the name of the file you want to process
% filename='HF-GM-13_11_19_13.xlsx';
sheet=3;                                    %number of the page containing the data to process

%method used for the processing
% method_Hay='beginning';                     %fit the parabols from the start
method_Hay='interval';                        %fit the parabols on moving interval
npoints=30;                                 %minimum number of points in the interval
ninter=30;                                  %maximum size interval= ninter+npoints


%% Load data from excel file
data = xlsread(filename, sheet);

%remove unnecessary columns
%BEWARE: depending on your Matlab version the xlsread function might not
%read the data propely. It is your job to ensure that the matrix called data has 
%times in the first column and pressures in the second one.
if size(data,2)>4
    data(:,5:end)=[];
end

if size(data,2)==4
    data(:,1)=[];
end

%remove header
while 1
   if sum(isnan(data(1,:)'))>0
       data(1,:)=[];
   else
       break
   end
end

%change _ with - in filename
matfilename=strrep(filename, '_', '-');
matfilename(end-4:end)=[];

%plot data
plot(data(:,1), data(:,2), '-*'),
xlabel('Time [min]')
ylabel('Pressure [Psia]')
title(['Pressure vs Time from ', matfilename])

%You should basically use this plot to create the descriptions of the
%events

%% Load event descriptions
filenamedescr=[filename(1:end-5),'_events.xlsx'];

%Check if the events have been described in an excel sheet and if it has
%been, it returns the number of events that are described
if exist(filenamedescr, 'file')==2
    mata = xlsread(filenamedescr, sheet);
    if isempty(mata)
        disp(['The file: ',filenamedescr,' exists but the sheet number ', num2str(sheet),' is empty']);
        return
    end
else
    disp(['Please provide a file called: ',filenamedescr,' that descibes the events']);
    return
end


% END OF PART 1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Hayashi
N_inv=200;                      %max number of parabols investigated
n_past=10;                      
valf=0.1;                       %threshold for the pressure difference when looking
                                %for the point where the data and the
                                %parabolic fit split

n_phase=size(mata,1);
for m=1:n_phase
    INF=mata(m,2);                      % event starting point
    SUP=mata(m,3);                      % event end point
    pt=mata(m,4);                       % pressure threshold
    
    %find the indices corresponding to the INF and SUP times
    inf=find(data(:,1)>=INF & data(:,1)<INF+1,1);
    sup=find(data(:,1)>=SUP & data(:,1)<SUP+1,1);
    tot=sup-inf+1;
    
    %Switch pressure and time axis to show that the parabol is fitted on t(P)
    zrt=zeros(tot,2);
    zrt(:,1)=data(inf:sup,2);
    zrt(:,2)=data(inf:sup,1);
    
    %Find the point from where we must perform the fitting.
    ndt=find((zrt(:,1)-pt)<=0,1);
    
    %Smooth the portion of the curve we want to approximate by a parabol
    p = polyfit( zrt(ndt:end,2), zrt(ndt:end,1), 23);
    zrt(ndt:end,1)=polyval(p,zrt(ndt:end,2));
    
    %Perform the fitting and store its result in mat. mat is made of the
    %coefficients (a,b,c) of the parabole (a*x^2+b*x+c) and there is also
    %the rmse in addition
    pressure_npoints=zeros(ninter,1);
    for j=0:2:ninter
        np=npoints+j;
        switch method_Hay
            case 'beginning'        %Fit data from the start point ndt. Iteratively add one point 
                                    %and recompute the fit.
                if N_inv> tot-ndt-2
                    n_inv=tot-ndt-2;
                else
                    n_inv=N_inv;
                end
                mat=zeros(n_inv,4);
                for i=2:n_inv-1         %n_inv is the max number of parabols tested.
                    if ndt+i<=tot
                        fitcoef = polyfit(zrt(ndt:ndt+i,1),zrt(ndt:ndt+i,2),2);
                        if fitcoef(1)>=0 && fitcoef(2)<=0 && fitcoef(3)>=0
                            mat(i+1,1:3)=fitcoef;
                            temp=polyval(fitcoef, zrt(ndt:ndt+i,1));
                            mat(i+1,4)=sqrt( sum( (temp(:)-zrt(ndt:ndt+i,2)).^2 )/(i+1) );
                        end
                    else
                        break
                    end
                end

            case 'interval'         %Fit the data on a fixed size moving interval.
                if N_inv> tot-ndt-np
                    n_inv=tot-ndt-np;
                else
                    n_inv=N_inv;
                end
                mat=zeros(n_inv,4);
                for i=np:n_inv-1
                    if ndt+i<=tot
                        fitcoef = polyfit(zrt(ndt+i-np:ndt+i-1,1),zrt(ndt+i-np:ndt+i-1,2),2);
                        if fitcoef(1)>=0 && fitcoef(2)<=0 && fitcoef(3)>=0
                            mat(i+1,1:3)=fitcoef;
                            temp=polyval(fitcoef, zrt(ndt+i-np:ndt+i-1,1));
                            mat(i+1,4)=sqrt( sum( (temp(:)-zrt(ndt+i-np:ndt+i-1,2)).^2 )/np );
                        end
                    else
                        break
                    end
                end
        end

        % Find the parabol that fits best the ploynomial data 
        % It must minimise the rmse
        switch method_Hay
            case 'beginning'
                I_min=mata(m,5);
            case 'interval'
                I_min=np+1;
        end
        [C,I]=min(mat(I_min:end,4));
        I=I+I_min-1;
        
        %file names and titles
        switch method_Hay
            case 'beginning'
                title_str1=[matfilename,' Phase ',num2str(m)];
                filename1=[matfilename,'_phase',num2str(m),'_begin_hayashi.jpg'];
                filename2=[matfilename,'_phase',num2str(m),'_begin_RMSE.jpg'];
            case 'interval'
                title_str1=[matfilename,' Phase ',num2str(m), ' using ', num2str(np), ' points for the interval'];
                filename1=[matfilename,'_phase',num2str(m),'_inter',num2str(np),'_hayashi.jpg'];
                filename2=[matfilename,'_phase',num2str(m),'_inter',num2str(np),'_RMSE.jpg'];
        end

        % plot the rmse
        h=figure;
        plot(1:n_inv,mat(1:n_inv,4))
        title(title_str1)
        xlabel('Number of points or interval location')
        ylabel('RMSE')
        legend({['I=',num2str(I),' points']});
        print(h, '-djpeg', filename2);
         close(h);

        % Reconstruct the best fitting parabol
        y=zeros(tot,1);
        for ii=ndt-1:tot
            y(ii,1)=mat(I,1)*zrt(ii,1)^2+mat(I,2)*zrt(ii,1)+mat(I,3);
        end

        %Now that we know which parabol to consider we look for the point where
        %the fitting stops. This corresponds to Hayashi's closing pressure.
        for i=ndt+n_past:tot            %n_past is used to jump some of the first results and start searching for the 
                                        %closing pressure further from the
                                        %beginning
            if zrt(i,2)-y(i,1)>valf
                Pmax=zrt(i-1,1);
                pressure_npoints(j+1)=Pmax;
                break
            end
        end

        h=figure;
        plot(zrt(:,2), zrt(:,1)), hold on
        plot(y(ndt-1:tot,1), zrt(ndt-1:tot,1),'r'), hold on
        hold off
        leg_string=strcat(['Hayashi, Pmax =',num2str(round(Pmax)),' Psi, t@Pmax = ',num2str(round(zrt(i-1,2)*10)/10),' min']);
        legend({'Data',leg_string})
        xlabel('Time [Min]')
        ylabel('Pressure [Psi]')
        title(title_str1)
        print(h, '-djpeg', filename1);          %print the figure and saves it in jpeg format under filename1 in the current folder
        close(h);
    end
end

figure
plot((0:2:ninter)+npoints, pressure_npoints(1:2:end))
title('Closing pressure as a function of interval size')
ylabel('Closing pressure')
xlabel('Interval size')

%END OF PART 2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Aamodt
valf=0.005;          %threshold for the difference between the fit and the data

for m=1:n_phase
    INF=mata(m,2);                                  % event starting point
    SUP=mata(m,3);                                  % event end point
    pw=mata(m,6);                                   %well  pressure computed from depth (rho*g*z)
    TRA=mata(m,7);          %TRA is the time from which we fit data. Basically we will fit the data
                            %situated between TRA and SUP.
                            
    %find indices corresponding to the times
    inf=find(data(:,1)>=INF & data(:,1)<INF+1,1,'first');
    sup=find(data(:,1)>=SUP & data(:,1)<SUP+1,1,'first');
    tra=find(data(:,1)>=TRA & data(:,1)<TRA+1,1,'first');
    tot=sup-inf+1;
    
    %define title and file names
    title_str1=[matfilename,' Phase ',num2str(m)];
    filename=[matfilename,'_phase',num2str(m),'_aamodt.jpg'];
    
    zt=data(inf:sup, 1:2);
    
    %compute the logarithm and perform the linear fit between tra and sup
    logpre=log(zt(:,2)-pw);
    fitcoef = polyfit(zt(tra-inf:sup-inf,1), logpre(tra-inf:sup-inf,1),1);
    
    %reconstruct the fit
    y=polyval(fitcoef, zt(:,1));
    difffit=logpre-y;
    
    %find position where the fit is not relevant anymore. This position
    %corresponds to Aamodt's closing pressure.
    for i=tra-inf:-1:1
        if ( abs(difffit(i,1)) > valf)
            ithresh=i+1;
            pthresh=zt(ithresh,2);
            break
        end
    end

    h=figure;
    plot(zt(:, 1), logpre(:, 1))
    hold on
    plot(zt(:, 1), y(:, 1),'r')
    title(title_str1)
    leg_string=strcat(['Aamodt / Pmin = ',num2str(floor(pthresh)),...
        ' Psi / t@Pmin = ',num2str(floor(zt(ithresh,1)*10)/10),' min']);
    legend({['Log(P-Pw) , Pw =',num2str(round(pw)),'Psi'],leg_string});
    xlabel('Time [min]')
    ylabel('Log(P-Pw) [log(Psi)]')
    hold off
    print(h, '-djpeg', filename);
%     close(h)
end

